import ssl
import certifi
import aiohttp

import logging

import asyncio
from . import loggers

from typing import List, Dict, Union, Optional
from .exceptions import *

logging.basicConfig(level=logging.INFO)

class Entities:
    def __init__(self, obj):
        self.type = obj.get("type", None)
        self.offset = obj.get("offset", None)
        self.length = obj.get("length", None)
        self.__user = obj.get("user", None)
        self.from_user = From_user(user_id=self.__user.get("id", None), fullname=self.__user.get("first_name", None), username=self.__user.get("username", None), language_code=self.__user.get("language_code", None), is_bot=self.__user.get("is_bot", None))

class From_user:
    def __init__(self, fullname: str, user_id: int, username: str, is_bot: bool, language_code: str):
        self.full_name = fullname
        self.id = user_id
        self.username = username
        self.is_bot = is_bot
        self.language_code = language_code
    
class Chat:
    def __init__(self, id: int):
        self.id = id

class Reply_to_message:
    def __init__(self, full_name: str, user_id: int, message_id: int, username: str, is_bot: bool, language_code: str,
                 token: str, chat_id: int):
        self.__token = token
        self.__url = f"https://api.telegram.org/bot{self.__token}/"
        self.full_name = full_name
        self.user_id = user_id
        self.message_id = message_id
        
        self.__chat_id = chat_id
        
        self.from_user = From_user(fullname=full_name, user_id=user_id, username=username, is_bot=is_bot, language_code=language_code)
        
        self.session = None
        self.ssl_context = ssl.create_default_context(cafile=certifi.where())
    
    async def start_session(self):
        if not self.session:
            self.session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=self.ssl_context))

    async def close_session(self):
        if self.session:
            await self.session.close()
            self.session = None
    
    async def delete_message(self) -> bool:
        await self.start_session()
        payload = {
            "chat_id": self.__chat_id,
            "message_id": self.message_id,
        }
        try:
            async with self.session.post(self.__url + "deleteMessage", json=payload) as response:
                if response.status == 200:
                    data = await response.json()
                    return True
                else:
                    raise TelegramBadRequest((await response.json()).get("description"))
        except Exception as e:
            loggers.event.error(f"{e}")
            return False
        finally:
            await self.close_session()
    
    async def reply(self, message: str, parse_mode: str="HTML", reply_markup=None):
        if not isinstance(parse_mode, str):
            loggers.event.error(f"Expected 'parse_mode' to be a string, got {type(parse_mode).__name__}")
            return False
        if not isinstance(message, str):
            loggers.event.error(f"Expected 'message' to be a string, got {type(message).__name__}")
            return False
        if message == "" and message == None:
            loggers.event.error("The message is empty.")
            return False
        await self.start_session()
        
        payload = {
            "chat_id": self.__chat_id,
            "text": message,
            "reply_to_message_id": self.message_id,
            "parse_mode": parse_mode
        }

        if reply_markup:
            payload["reply_markup"] = reply_markup

        try:
            async with self.session.post(self.__url + "sendMessage", json=payload) as response:
                response.raise_for_status()
                data = await response.json()
                loggers.event.info("The message has been sent successfully.")
                return True
        except Exception as e:
            loggers.event.error(f"{e}")
            return False
        finally:
            await self.close_session()
    
    async def reply_photo(self, file_path: str=None, url_photo: str=None, caption: str = None, parse_mode: str="HTML", reply_markup=None):
        if not isinstance(parse_mode, str):
            raise ValueError("The 'parse_mode' parameter cannot be None or not a string.")
        
        if not isinstance(url_photo, str):
            raise ValueError("The 'url_photo' parameter cannot be None or not a string.")
        await self.start_session()
        
        try:
            if file_path:
                # Чтение файла и создание FormData
                with open(file_path, "rb") as photo_file:
                    form_data = aiohttp.FormData()
                    form_data.add_field("chat_id", str(self.__chat_id))
                    form_data.add_field("photo", photo_file, filename=file_path.split("/")[-1])
                    form_data.add_field("parse_mode", parse_mode)
                    form_data.add_field("reply_to_message_id", self.message_id)
            else:
                form_data = aiohttp.FormData()
                form_data.add_field("chat_id", str(self.__chat_id))
                form_data.add_field("photo", url_photo)
                form_data.add_field("parse_mode", parse_mode)
                form_data.add_field("reply_to_message_id", self.message_id)

            if caption:
                form_data.add_field("caption", caption)

            if reply_markup:
                form_data.add_field("reply_markup", reply_markup)
                
            # Отправка запроса
            async with self.session.post(f"{self.__url}sendPhoto", data=form_data) as response:
                response.raise_for_status()  # Бросает исключение при HTTP-ошибке
                loggers.event.info("The photo was successfully sent.")
                data = await response.json()
                await self.close_session()
                return True
        
        except aiohttp.ClientError as e:
            loggers.event.error(f"ERROR: {e}")
            return False
        
        except FileNotFoundError:
            loggers.event.error(f"ERROR: File not found: {file_path}")
            return False

class MessageObject:
    def __init__(self, chat_id: int, message_id: int, fullname: str, user_id: int, username: str, is_bot: bool,
                 token: str, type_chat: str, message_text: str,
                 reply_to_message_fullname: str=None, reply_to_message_user_id: int=None, reply_to_message_message_id: int=None, reply_is_bot: bool=None,
                 language_code: str=None, reply_language_code: str=None):
        self.__chat_id = chat_id
        
        self.message_id = message_id
        self.text = message_text
        self.type = type_chat
        if reply_to_message_message_id:
            self.reply_to_message = Reply_to_message(full_name=reply_to_message_fullname, user_id=reply_to_message_user_id, message_id=reply_to_message_message_id, username=username, is_bot=reply_is_bot, language_code=reply_language_code,
                                                    token=token, chat_id=self.__chat_id)
        else:
            self.reply_to_message = None
        self.from_user = From_user(fullname=fullname, user_id=user_id, username=username, is_bot=is_bot, language_code=language_code)
        self.chat = Chat(chat_id)
        self.__token = token
        self.__url = "https://api.telegram.org/bot" + self.__token + "/"
        
        self.session = None
        self.ssl_context = ssl.create_default_context(cafile=certifi.where())
    
    async def start_session(self):
        if not self.session:
            self.session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=self.ssl_context))

    async def close_session(self):
        if self.session:
            await self.session.close()
            self.session = None
    
    async def answer(self, message: str, parse_mode: str="HTML", reply_markup=None):
        if not isinstance(parse_mode, str):
            raise ValidationError(f"The 'parse_mode' parameter cannot be None or not a string.")
        if not isinstance(message, str):
            raise ValidationError(f"The 'message' parameter cannot be None or not a string.")
        if message == "" and message == None:
            raise ValidationError("The message is empty.")
        await self.start_session()
        
        payload = {
            "chat_id": self.__chat_id,
            "text": message,
            "parse_mode": parse_mode
        }
        
        if reply_markup:
            payload["reply_markup"] = reply_markup
        
        try:
            async with self.session.post(self.__url + "sendMessage", json=payload) as response:
                response.raise_for_status()  # Бросает исключение для HTTP-ошибок
                data = await response.json()
                loggers.event.info("The message has been sent successfully.")
                return True
        except Exception as e:
            loggers.event.error(f"{e}")
            return False
        finally:
            await self.close_session()
    
    async def reply(self, message: str, parse_mode: str="HTML", reply_markup=None):
        if not isinstance(parse_mode, str):
            loggers.event.error(f"Expected 'parse_mode' to be a string, got {type(parse_mode).__name__}")
            return False
        if not isinstance(message, str):
            loggers.event.error(f"Expected 'message' to be a string, got {type(message).__name__}")
            return False
        if message == "" and message == None:
            loggers.event.error("The message is empty.")
            return False
        await self.start_session()
        
        payload = {
            "chat_id": self.__chat_id,
            "text": message,
            "reply_to_message_id": self.message_id,
            "parse_mode": parse_mode
        }

        if reply_markup:
            payload["reply_markup"] = reply_markup

        try:
            async with self.session.post(self.__url + "sendMessage", json=payload) as response:
                response.raise_for_status()
                data = await response.json()
                loggers.event.info("The message has been sent successfully.")
                return True
        except Exception as e:
            loggers.event.error(f"{e}")
            return False
        finally:
            await self.close_session()
    
    async def delete_message(self) -> bool:
        await self.start_session()
        payload = {
            "chat_id": self.__chat_id,
            "message_id": self.message_id,
        }
        try:
            async with self.session.post(self.__url + "deleteMessage", json=payload) as response:
                if response.status == 200:
                    data = await response.json()
                    return True
                else:
                    raise TelegramBadRequest((await response.json()).get("description"))
        except Exception as e:
            loggers.event.error(f"{e}")
            return False
        finally:
            await self.close_session()
    
    async def reply_photo(self, file_path: str=None, url_photo: str=None, caption: str = None, parse_mode: str="HTML", reply_markup=None):
        if not isinstance(parse_mode, str):
            raise ValueError("The 'parse_mode' parameter cannot be None or not a string.")

        if not isinstance(url_photo, str):
            raise ValueError("The 'url_photo' parameter cannot be None or not a string.")
        await self.start_session()
        
        try:
            if file_path:
                # Чтение файла и создание FormData
                with open(file_path, "rb") as photo_file:
                    form_data = aiohttp.FormData()
                    form_data.add_field("chat_id", str(self.__chat_id))
                    form_data.add_field("photo", photo_file, filename=file_path.split("/")[-1])
                    form_data.add_field("parse_mode", parse_mode)
                    form_data.add_field("reply_to_message_id", self.message_id)
            else:
                form_data = aiohttp.FormData()
                form_data.add_field("chat_id", str(self.__chat_id))
                form_data.add_field("photo", url_photo)
                form_data.add_field("parse_mode", parse_mode)
                form_data.add_field("reply_to_message_id", self.message_id)

            if caption:
                form_data.add_field("caption", caption)

            if reply_markup:
                form_data.add_field("reply_markup", reply_markup)
                
            # Отправка запроса
            async with self.session.post(f"{self.__url}sendPhoto", data=form_data) as response:
                response.raise_for_status()  # Бросает исключение при HTTP-ошибке
                loggers.event.info("The photo was successfully sent.")
                data = await response.json()
                await self.close_session()
                return True
        
        except aiohttp.ClientError as e:
            loggers.event.error(f"ERROR: {e}")
            return False
        
        except FileNotFoundError:
            loggers.event.error(f"ERROR: File not found: {file_path}")
            return False
    
    async def answer_photo(self, file_path: str=None, url_photo: str=None, caption: str = None, parse_mode: str="HTML", reply_markup=None):
        if not isinstance(parse_mode, str):
            loggers.event.error(f"Expected 'parse_mode' to be a string, got {type(parse_mode).__name__}")
            return False

        if not isinstance(url_photo, str):
            loggers.event.error(f"Expected 'url_photo' to be a string, got {type(url_photo).__name__}")
            return False
        await self.start_session()
        
        try:
            if file_path:
                # Чтение файла и создание FormData
                with open(file_path, "rb") as photo_file:
                    form_data = aiohttp.FormData()
                    form_data.add_field("chat_id", str(self.__chat_id))
                    form_data.add_field("photo", photo_file, filename=file_path.split("/")[-1])
                    form_data.add_field("parse_mode", parse_mode)
            else:
                form_data = aiohttp.FormData()
                form_data.add_field("chat_id", str(self.__chat_id))
                form_data.add_field("photo", url_photo)
                form_data.add_field("parse_mode", parse_mode)

            if caption:
                form_data.add_field("caption", caption)

            if reply_markup:
                form_data.add_field("reply_markup", reply_markup)
            
            # Отправка запроса
            async with self.session.post(f"{self.__url}sendPhoto", data=form_data) as response:
                response.raise_for_status()  # Бросает исключение при HTTP-ошибке
                loggers.event.info("The photo was successfully sent.")
                data = await response.json()
                await self.close_session()
                return True
        
        except aiohttp.ClientError as e:
            loggers.bot.error(f"ERROR: {e}")
            return False
        
        except FileNotFoundError:
            loggers.bot.error(f"ERROR: File not found: {file_path}")
            return False
    
    async def edit_text(self, text: str):
        if not isinstance(text, str):
            raise ValidationError(f"Expected 'text' to be a string, got {type(text).__name__}")
            return
        await self.start_session()
        payload = {
            "chat_id": self.__chat_id,
            "message_id": self.message_id,
            "text": text
        }
        
        try:
            async with self.session.post(self.__url + "editMessageText", json=payload) as response:
                if response.status == 200:
                    data = (await response.json()).get("result")
                    data_from = data.get("from")
                    data_chat = data.get("chat")
                    return MessageObject(message_id=int(data.get("message_id")), fullname=data_from.get("fullname"), username=data_from.get("username"),
                                        is_bot=data_from.get("is_bot"), message_text=data.get("text"), user_id=data_from.get("id"),
                                        chat_id=data_chat.get("id"), type_chat=data_chat.get("type"), token=self.__token)
                else:
                    raise TelegramBadRequest((await response.json()).get("description"))
        except aiohttp.ClientError as e:
            loggers.event.error(f"ERROR: {e}")
        finally:
            await self.close_session()
    
    async def answer_dice(self, emoji: Optional[str]="🎲", parse_mode: str="HTML", reply_markup=None):
        if not isinstance(emoji, str):
            # loggers.types.error(f"Expected 'parse_mode' to be a string, got {type(parse_mode).__name__}")
            # loggers.types.error("ERROR")
            raise ValueError("The 'emoji' parameter cannot be None or not a string.")
        if not isinstance(parse_mode, str):
            # loggers.types.error("ERROR")
            raise ValueError("The 'parse_mode' parameter cannot be None or not a string.")
        
        await self.start_session()
        
        payload = {
            "chat_id": self.__chat_id,
            "emoji": emoji,
            "parse_mode": parse_mode
        }

        if reply_markup:
            payload["reply_markup"] = reply_markup

        try:
            async with self.session.post(self.__url + "sendDice", json=payload) as response:
                response.raise_for_status()
                data = await response.json()
                # loggers.types.info("The message has been sent successfully.")
                return Dice(data.get("result"))
        except Exception as e:
            loggers.event.error(f"ERROR: {e}")
        finally:
            await self.close_session()

class CallbackQuery:
    def __init__(self, obj: dict, token: str):
        self.id = obj.get("id", None)
        self.message_json = obj.get("message", {})
        self.message = MessageObject(
    is_bot=self.message_json.get("from", {}).get("is_bot", None),
    reply_is_bot=self.message_json.get("reply_to_message", {}).get("from", {}).get("is_bot", None),
    language_code=self.message_json.get("from", {}).get("language_code", None),
    reply_language_code=self.message_json.get("reply_to_message", {}).get("from", {}).get("language_code", None),
    message_id=self.message_json.get("message_id", None),
    username=self.message_json.get("from", {}).get("username", None),
    chat_id=self.message_json.get("chat", {}).get("id", None),
    fullname=self.message_json.get("from", {}).get("first_name", None),
    user_id=self.message_json.get("from", {}).get("id", None),
    reply_to_message_fullname=self.message_json.get("reply_to_message", {}).get("from", {}).get("first_name", None),
    reply_to_message_user_id=self.message_json.get("reply_to_message", {}).get("from", {}).get("id", None),
    reply_to_message_message_id=self.message_json.get("reply_to_message", {}).get("message_id", None),
    token=token,
    type_chat=self.message_json.get("chat", {}).get("type", None),
    message_text=self.message_json.get("text", None)
)
        # self.message = MessageObject(message_id=self.message.get("message_id", None), chat_id=obj.get("message", None).get("chat", None).get("id", None), fullname=self.message.get("from", None).get("first_name", None), user_id=self.message.get("from", None).get("id", None), reply_to_message_fullname=self.message.get("reply_to_message", None).get("from", None).get("first_name", None), reply_to_message_user_id=self.message.get("reply_to_message", None).get("from", None).get("id", None), reply_to_message_message_id=self.message.get("reply_to_message", None).get("message_id", None), token=obj.get("token", None), type_chat=self.message.get("chat", None).get("type", None), message_text=self.message.get("text", None))
        self.chat_instance = obj.get("chat_instance", None)
        self.data = obj.get("data", None)
        self.entities=Entities(self.message_json.get("entities", {})[0])
        
        self.__token = token
        self.__url = "https://api.telegram.org/bot" + self.__token + "/"
        
        self.session = None
        self.ssl_context = ssl.create_default_context(cafile=certifi.where())
    
    async def start_session(self):
        if not self.session:
            self.session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=self.ssl_context))

    async def close_session(self):
        if self.session:
            await self.session.close()
            self.session = None
    
    async def answer(self, text: str="", show_alert: bool=False, cache_time: int=0):
        if not isinstance(text, str):
            raise ValueError("The 'text' parameter cannot be None or not a string.")
        if not isinstance(show_alert, bool):
            raise ValueError("The 'show_alert' parameter cannot be None or not a boolean.")
        if not isinstance(cache_time, int):
            raise ValueError("The 'cache_time' parameter cannot be None or not a integer.")

        await self.start_session()

        payload = {
            "callback_query_id": self.id,
            "text": text,
            "show_alert": show_alert,
            "cache_time": cache_time
        }
        try:
            async with self.session.post(self.__url + "answerCallbackQuery", json=payload) as response:
                response.raise_for_status()
                data = await response.json()
                return True
        except Exception as e:
            raise TelegramBadRequest(e)
        finally:
            await self.close_session()

class Permissions:
    def __init__(self, obj: dict):
        can_send_messages: bool = obj.get("can_send_messages", None)
        can_send_media_messages: bool = obj.get("can_send_media_messages", None)
        can_send_audios: bool = obj.get("can_send_audios", None)
        can_send_documents: bool = obj.get("can_send_documents", None)
        can_send_photos: bool = obj.get("can_send_photos", None)
        can_send_videos: bool = obj.get("can_send_videos", None)
        can_send_video_notes: bool = obj.get("can_send_video_notes", None)
        can_send_voice_notes: bool = obj.get("can_send_voice_notes", None)
        can_send_polls: bool = obj.get("can_send_polls", None)
        can_send_other_messages: bool = obj.get("can_send_other_messages", None)
        can_add_web_page_previews: bool = obj.get("can_add_web_page_previews", None)
        can_change_info: bool = obj.get("can_change_info", None)
        can_invite_users: bool = obj.get("can_invite_users", None)
        can_pin_messages: bool = obj.get("can_pin_messages", None)
        can_manage_topics: bool = obj.get("can_manage_topics", None)

class GetChat:
    def __init__(self, obj: dict):
        self.id: int = obj.get("id", None)
        self.title: str = obj.get("title", None)
        self.type: str = obj.get("type", None)
        self.invite_link: str = obj.get("invite_link", None)
        self.permissions = Permissions(obj.get("permissions", None))
        self.join_to_send_messages: bool = obj.get("join_to_send_messages", None)
        self.max_reaction_count: int = obj.get("max_reaction_count", None)
        self.accent_color_id: int = obj.get("accent_color_id", None)

class CommandObject:
    def __init__(self, text_message: str):
        self.args = " ".join(text_message.split()[1:])
        self.args_list = text_message.split()[1:]

class Dice:
    def __init__(self, obj: dict):
        self.__dice = obj.get("dice", {})
        self.emoji = self.__dice.get("emoji", None)
        self.value = self.__dice.get("value", None)

class GetMe:
    def __init__(self, obj: dict):
        self.id: int = obj.get("id", None)
        self.is_bot: bool = obj.get("is_bot", None)
        self.first_name: str = obj.get("first_name", None)
        self.last_name: str = obj.get("last_name", None)
        self.username: str = obj.get("username", None)
        self.language_code: str = obj.get("language_code", None)
        self.can_join_groups: bool = obj.get("can_join_groups", None)
        self.can_read_all_group_messages: bool = obj.get("can_read_all_group_messages", None)
        self.supports_inline_queries: bool = obj.get("supports_inline_queries", None)



"""
:authors: Bogdan Boris
:license: Apache License, Version 2.0, see LICENSE file

:copyright: (c) 2025 Bogdan Boris
"""

from .bot import *
from .html import *
from .loggers import *
from .types import *
from .keyboards import *
from .exceptions import *

__author__ = "Bogdan Boris"
__version__ = "0.1.1"
__email__ = "gdrghdhgddy@gmail.com"


from typing import Optional

class AioTeleError(Exception):
    """
    Base exception for all aiogram errors.
    """


class DetailedAioTeleError(AioTeleError):
    """
    Base exception for all aiogram errors with detailed message.
    """

    url: Optional[str] = None

    def __init__(self, message: str) -> None:
        self.message = message

    def __str__(self) -> str:
        message = self.message
        if self.url:
            message += f"\n(background on this error at: {self.url})"
        return message

    def __repr__(self) -> str:
        return f"{type(self).__name__}('{self}')"

class TelegramAPIError(DetailedAioTeleError):
    """
    Base exception for all Telegram API errors.
    """
    
    label: str = "Telegram server says"
    
    def __init__(self, message: str) -> None:
        super().__init__(message=message)
    
    def __str__(self) -> str:
        original_message = super().__str__()
        return f"{self.label} - {original_message}"

class TelegramBadRequest(TelegramAPIError):
    """
    The standard exception for TelegramBadRequest errors with a description
    """

class ValidationError(DetailedAioTeleError):
    """
    Input data validation error.
    """

class TelegramConflictError(TelegramAPIError):
    """
    The standard exception for TelegramConflictError errors with a description
    """

class TelegramNetworkError(DetailedAioTeleError):
    """
    Network communication error.
    """



import aiohttp.client_exceptions
import aiohttp.http_exceptions
import aiohttp.web
from aiotele.types import MessageObject, GetChat, CommandObject, GetMe, CallbackQuery
import aiohttp
import asyncio

from aiotele import loggers

import ssl
import certifi

import inspect
from contextlib import suppress

from typing import List, Optional
from .exceptions import *

class CommandHandler:
    def __init__(self, token: str):
        self.commands = {}
        self.__token = token
        self.default_handler = None  # Хендлер для любого сообщения

    def command(self, command: str = None, commands: List[str] = None, prefix: str = None):
        def wrapper(func):
            if command is None and commands is None:
                self.default_handler = func  # Устанавливаем обработчик для любого текста
            elif command is None and commands:
                for _command in commands:
                    if prefix != None:
                        for i in prefix:
                            _command = i + _command
                            self.commands[_command] = func
            elif isinstance(command, str):
                if prefix != None:
                    for i in prefix:
                        _command = i + command
                        self.commands[_command] = func
                else:
                    _command = command
                    self.commands[_command] = func
            return func
        return wrapper

    async def handle(self, update, bot):
        message = update.get("message", {})
        text = message.get("text", "")
        command = text.split(" ")[0]

        # Проверяем, есть ли обработчик для команды или общий обработчик для любого сообщения
        handler = self.commands.get(command, self.default_handler)

        if handler:
            from_info = message.get("from", {})
            username = from_info.get("username", None)
            user_id = from_info.get("id", 0)
            full_name = f"{from_info.get('first_name', '')} {from_info.get('last_name', '')}".strip()
            chat = message.get("chat", {})
            type_chat = chat.get("type", "None")
            chat_id = chat.get("id", 0)
            message_id = message.get("message_id", 0)

            reply_to_info = message.get("reply_to_message", {})
            reply_to_message_id = reply_to_info.get("message_id", None)
            reply_to_from_info = reply_to_info.get("from", {})
            reply_is_bot = reply_to_from_info.get("is_bot", None)
            reply_to_user_id = reply_to_from_info.get("id", None)
            reply_to_full_name = f"{reply_to_from_info.get('first_name', '')} {reply_to_from_info.get('last_name', '')}".strip()
            
            msg_obj = MessageObject(
                fullname=full_name,
                is_bot=from_info.get("is_bot", None),
                reply_is_bot=reply_is_bot,
                language_code=from_info.get("language_code", None),
                reply_language_code=reply_to_from_info.get("language_code", None),
                username=username,
                user_id=user_id,
                chat_id=chat_id,
                message_id=message_id,
                reply_to_message_fullname=reply_to_full_name,
                reply_to_message_user_id=reply_to_user_id,
                reply_to_message_message_id=reply_to_message_id,
                token=self.__token,
                type_chat=type_chat,
                message_text=text
            )
            command_obj = CommandObject(
                text_message=text,
            )
            sig = inspect.signature(handler)
            params = len(sig.parameters)
            if params == 1:
                await handler(msg_obj)
            else:
                await handler(msg_obj, command_obj)

class CallbackDataHandler:
    def __init__(self, token: str):
        self.commands = {}
        self.__token = token
        self.default_handler = None  # Хендлер для любого сообщения

    def callback(self, command: str = None):
        def wrapper(func):
            if command is None:
                self.default_handler = func  # Устанавливаем обработчик для любого текста
            elif command is None:
                _command = i + _command
                self.commands[_command] = func
            elif isinstance(command, str):
                self.commands[command] = func
            return func
        return wrapper

    async def handle(self, update, bot):
        message = update.get("message", {})
        callback = update.get("callback_query", None)
        command = callback.get("data", None)

        # Проверяем, есть ли обработчик для команды или общий обработчик для любого сообщения
        handler = self.commands.get(command, self.default_handler)

        if handler:
            callback_obj = CallbackQuery(callback, self.__token)
            await handler(callback_obj)

class Bot:
    def __init__(self, TOKEN: str):
        self.__token = TOKEN
        self.__url = f"https://api.telegram.org/bot{self.__token}/"
        self.__message_handler = CommandHandler(token=self.__token)
        self.__callback_handler = CallbackDataHandler(token=self.__token)
        self.update_offset = 0
        self.session = None
        self.ssl_context = ssl.create_default_context(cafile=certifi.where())
    
    def message_handler(self, command: str = None, commands: List[str] = None, prefix: str = None):
        return self.__message_handler.command(command, commands, prefix)

    def callback_handler(self, command: str = None):
        return self.__callback_handler.callback(command)

    async def start_session(self):
        if not self.session:
            self.session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=self.ssl_context))

    async def close_session(self):
        if self.session:
            await self.session.close()
            self.session = None

    async def send_message(self, chat_id: int, message: str, reply_to_message_id: int=None, parse_mode: str="HTML", reply_markup=None):
        if not isinstance(parse_mode, str):
            loggers.event.error(f"Expected 'parse_mode' to be a string, got {type(parse_mode).__name__}")
            return False
        if not isinstance(message, str):
            loggers.event.error(f"Expected 'message' to be a string, got {type(message).__name__}")
            return False
        if message == "" and message == None:
            loggers.event.error("The message is empty.")
            return False
        await self.start_session()
        
        payload = {
            "chat_id": chat_id,
            "text": message,
            "parse_mode": parse_mode
        }
        
        if reply_markup:
            payload["reply_markup"] = reply_markup
        if reply_to_message_id:
            payload["reply_to_message_id"] = reply_to_message_id

        try:
            async with self.session.post(self.__url + "sendMessage", json=payload) as response:
                response.raise_for_status()  # Бросает исключение для HTTP-ошибок
                data = (await response.json()).get("result")
                loggers.event.info("The message has been sent successfully.")
                data_from = data.get("from")
                data_chat = data.get("chat")
                return MessageObject(message_id=int(data.get("message_id")), fullname=data_from.get("fullname"), username=data_from.get("username"),
                                     is_bot=data_from.get("is_bot"), message_text=data.get("text"), user_id=data_from.get("id"),
                                     chat_id=data_chat.get("id"), type_chat=data_chat.get("type"), token=self.__token)
        except Exception as e:
            raise ValidationError(f"{e}")
    
    async def send_photo(self, chat_id: int, file_path: str=None, url_photo: str=None, message_id: int=None, caption: str = None, parse_mode: str="HTML", reply_markup=None):
        if not isinstance(parse_mode, str):
            raise ValidationError("The 'parse_mode' parameter cannot be None or not a string.")
        
        if not isinstance(url_photo, str):
            raise ValidationError("The 'url_photo' parameter cannot be None or not a string.")
        await self.start_session()
        
        try:
            if file_path:
                # Чтение файла и создание FormData
                with open(file_path, "rb") as photo_file:
                    form_data = aiohttp.FormData()
                    form_data.add_field("chat_id", str(chat_id))
                    form_data.add_field("photo", photo_file, filename=file_path.split("/")[-1])
                    form_data.add_field("parse_mode", parse_mode)
            else:
                form_data = aiohttp.FormData()
                form_data.add_field("chat_id", str(chat_id))
                form_data.add_field("photo", url_photo)
                form_data.add_field("parse_mode", parse_mode)
            if caption:
                form_data.add_field("caption", caption)

            if reply_markup:
                form_data.add_field("reply_markup", reply_markup)
            
            if message_id:
                form_data.add_field("reply_to_message_id", message_id)
                
            # Отправка запроса
            async with self.session.post(f"{self.__url}sendPhoto", data=form_data) as response:
                response.raise_for_status()  # Бросает исключение при HTTP-ошибке
                loggers.event.info("The photo was successfully sent.")
                data = await response.json()
                return True
        
        except aiohttp.ClientError as e:
            loggers.event.error(f"ERROR: {e}")
            return False
        
        except FileNotFoundError:
            error_message = f"File not found: {file_path}"
            loggers.event.error(error_message)
            return False
    
    async def set_bot_name(self, name: Optional[str]=None, language_code: Optional[str]=None):
        if name == None:
            raise ValidationError("The name of the bot cannot be empty.")
        await self.start_session()
        
        payload = {
            "name": name,
        }
        if language_code:
            payload["language_code"] = language_code

        try:
            # Отправка запроса
            async with self.session.post(f"{self.__url}setMyName", json=payload) as response:
                if not (await response.json()).get("ok"):
                    loggers.bot.error(f"{(await response.json()).get("description")}")
                    return False
                response.raise_for_status()  # Бросает исключение при HTTP-ошибке
                loggers.bot.info("The bot's name has been successfully changed to %s", name)
                data = await response.json()
                return True
        except Exception as e:
            raise TelegramBadRequest(f"{e}")
    
    async def get_chat(self, chat_id: int):
        if not isinstance(chat_id, int):
            loggers.types.error(f"Expected 'chat_id' to be an integer, got {type(chat_id).__name__}")
            return
        await self.start_session()
        
        try:
            payload = {
                "chat_id": chat_id
            }
            # Отправка запроса
            async with self.session.post(f"{self.__url}getChat", json=payload) as response:
                response.raise_for_status()  # Бросает исключение при HTTP-ошибке
                loggers.bot.info("The message has been sent successfully.")
                data = await response.json()
                return GetChat(data.get("result"))
        except aiohttp.ClientError as e:
            loggers.bot.error(f"ERROR: {e}")
            return {"ok": False, "error": str(e)}
    
    async def get_updates(self):
        await self.start_session()
        async with self.session.get(self.__url + "getUpdates", params={"offset": self.update_offset}) as response:
            if response.status == 200:
                data = await response.json()
                for update in data.get("result", []):
                    if update.get("callback_query", None) != None:
                        self.update_offset = update["update_id"] + 1
                        loggers.event.info("Update has been successfully handled.")
                        await self.__callback_handler.handle(update, self)
                        break
                    
                    self.update_offset = update["update_id"] + 1
                    loggers.event.info("Update has been successfully handled.")
                    await self.__message_handler.handle(update, self)

    async def get_me(self):
        await self.start_session()
        async with self.session.get(self.__url + "getMe") as response:
            if response.status == 200:
                data = await response.json()
                return GetMe(data.get("result"))
            else:
                raise ValidationError((await response.json()).get("description"))
    
    async def edit_text(self, chat_id: int, message_id: int, text: str) -> MessageObject:
        if not isinstance(chat_id, int):
            raise ValidationError(f"Expected 'chat_id' to be an integer, got {type(chat_id).__name__}")
            return
        if not isinstance(message_id, int):
            raise ValidationError(f"Expected 'message_id' to be an integer, got {type(message_id).__name__}")
            return
        if not isinstance(text, str):
            raise ValidationError(f"Expected 'text' to be a string, got {type(text).__name__}")
            return
        await self.start_session()
        payload = {
            "chat_id": chat_id,
            "message_id": message_id,
            "text": text
        }
        
        async with self.session.post(self.__url + "editMessageText", json=payload) as response:
            if response.status == 200:
                data = (await response.json()).get("result")
                data_from = data.get("from")
                data_chat = data.get("chat")
                return MessageObject(message_id=int(data.get("message_id")), fullname=data_from.get("fullname"), username=data_from.get("username"),
                                     is_bot=data_from.get("is_bot"), message_text=data.get("text"), user_id=data_from.get("id"),
                                     chat_id=data_chat.get("id"), type_chat=data_chat.get("type"), token=self.__token)
            else:
                raise TelegramBadRequest((await response.json()).get("description"))
    
    async def delete_message(self, chat_id: int, message_id: int) -> bool:
        if not isinstance(chat_id, int):
            raise ValidationError(f"Expected 'chat_id' to be an integer, got {type(chat_id).__name__}")
            return
        if not isinstance(message_id, int):
            raise ValidationError(f"Expected 'message_id' to be an integer, got {type(message_id).__name__}")
            return
        await self.start_session()
        payload = {
            "chat_id": chat_id,
            "message_id": message_id,
        }
        
        async with self.session.post(self.__url + "deleteMessage", json=payload) as response:
            if response.status == 200:
                data = await response.json()
                return True
            else:
                raise TelegramBadRequest((await response.json()).get("description"))
    
    async def delete_webhook(self, drop_pending_updates: bool=False) ->  bool:
        if not isinstance(drop_pending_updates, bool):
            raise ValidationError(f"Expected 'drop_pending_updates' to be a boolean, got {type(drop_pending_updates).__name__}")
        await self.start_session()
        payload = {
            "drop_pending_updates": str(drop_pending_updates).lower()
        }
        
        url = f"{self.__url}deleteWebhook"
        async with self.session.post(url, params=payload) as response:
            if response.status == 200:
                data = await response.json()
                return True
            else:
                return False

    async def run(self) -> None:
        try:
            bot = await self.get_me()
            loggers.bot.info("Poll started")
            loggers.bot.info(f"Bot with the name '{bot.first_name}' and the username @{bot.username} has been launched")
            while True:
                await self.get_updates()
                await asyncio.sleep(1)
        except asyncio.CancelledError:
            pass
        except Exception:
            raise TelegramNetworkError("Couldn't connect to telegram bot. Check your internet connection if it doesn't help, check bot's token for correctness.")
        # except Exception:
        #     loggers.bot.error(f"Couldn't connect to telegram bot\nCheck your internet connection if it doesn't help, check bot's token for correctness.")
        finally:
            loggers.bot.info(f"Poll stopped")
            await self.close_session()



from io import open
from setuptools import setup, find_packages
 
version = "0.1.1"

with open("./aiotele/README.md", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="AioTele",
    version=version,
    
    author="Bogdan Boris",
    author_email="gdrghdhgddy@gmail.com",
    
    description=(
        "AioTele is a Python module for building Telegram bots using asyncio. "
        "It offers an intuitive API for handling updates, commands, and messaging, "
        "supporting both long polling and webhooks for scalable, high-performance bots."
    ),
    long_description=long_description,
    long_description_content_type="text/markdown",
    
    url="https://github.com/Bogdan-godot/AioTele",
    download_url="https://github.com/Bogdan-godot/AioTele/archive/v{}.zip".format(version),
    
    license="Apache License, Version 2.0, see LICENSE file",
    
    packages=find_packages(),  # Автоматический поиск пакетов
    include_package_data=True,
    install_requires=[
        "aiohttp>=3.8.1",
        "certifi>=2025.1.31"
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        "Topic :: Internet :: WWW/HTTP",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    keywords="telegram bot asyncio async telegram",
    python_requires=">=3.7",
)



import logging

bot = logging.getLogger("aiotele.bot")
types = logging.getLogger("aiotele.types")
event = logging.getLogger("aiotele.event")
middlewares = logging.getLogger("aiotele.middlewares")
webhook = logging.getLogger("aiotele.webhook")
scene = logging.getLogger("aiotele.scene")


class Button:
    def __init__(self, text: str, one_time_keyboard: bool = False, resize_keyboard: bool = True):
        self.text = text
        self.one_time_keyboard = one_time_keyboard
        self.resize_keyboard = resize_keyboard
    
    def __str__(self):
        keyboard = {
            "keyboard": [
                [self.text]
            ],
            "resize_keyboard": self.resize_keyboard,
            "one_time_keyboard": self.one_time_keyboard
        }
        return keyboard

class InlineButton:
    def __init__(self, text: str, callback_data: str):
        self.text = text
        self.callback_data = callback_data
    
    def __str__(self):
        keyboard = {
            "inline_keyboard": [
                [
                    {
                        "text": self.text,
                        "callback_data": self.callback_data
                    }
                ]
            ]
        }
        return keyboard

class MarkupButton:
    def __init__(self, resize_keyboard: bool = True, one_time_keyboard: bool = False):
        self.keyboards = {
            "keyboard": [],
            "resize_keyboard": resize_keyboard,
            "one_time_keyboard": one_time_keyboard
        }
    
    def add(self, button: Button):
        self.keyboards["keyboard"].append([button.text])
    
    def get(self):
        return self.keyboards

class MarkupButton:
    def __init__(self, resize_keyboard: bool = True, one_time_keyboard: bool = False):
        self.keyboards = {
            "keyboard": [],
            "resize_keyboard": resize_keyboard,
            "one_time_keyboard": one_time_keyboard
        }
    
    def add(self, button: Button):
        self.keyboards["keyboard"].append([button.text])
    
    def get(self):
        return self.keyboards

class MarkupButtonInline:
    def __init__(self):
        self.keyboards = {
            "inline_keyboard": [],
        }
    
    def add(self, button: InlineButton):
        self.keyboards["inline_keyboard"].append([{"text": button.text, "callback_data": button.callback_data}])
    
    def get(self):
        return self.keyboards